//
// ServiceEndpoint.cs
//
// Author:
//	Atsushi Enomoto <atsushi@ximian.com>
//
// Copyright (C) 2005 Novell, Inc.  http://www.novell.com
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
using System;
using System.Collections.Generic;
using System.ServiceModel.Channels;
using System.ServiceModel;

namespace System.ServiceModel.Description
{
	public class ServiceEndpoint
	{
		ContractDescription contract;
		Binding binding;
		EndpointAddress address;
		KeyedByTypeCollection<IEndpointBehavior> behaviors;
		Uri listen_uri;
		ListenUriMode listen_mode;
		string name;

		public ServiceEndpoint (ContractDescription contract)
			: this (contract, null, null)
		{
		}

		public ServiceEndpoint(ContractDescription contract,
			Binding binding, EndpointAddress address)
		{
			if (contract == null)
				throw new ArgumentNullException ("contract");

			this.contract = contract;
			this.binding = binding;
			this.address = address;
			behaviors = new KeyedByTypeCollection<IEndpointBehavior> ();
		}

		public KeyedByTypeCollection<IEndpointBehavior> Behaviors {
			get { return behaviors; }
		}

		public ContractDescription Contract {
			get { return contract; }
		}

		public EndpointAddress Address {
			get { return address; }
			set { address = value; }
		}

		public Binding Binding {
			get { return binding; }
			set { binding = value; }
		}

		public Uri ListenUri {
			get { return listen_uri; }
			set { listen_uri = value; }
		}

		public ListenUriMode ListenUriMode {
			get { return listen_mode; }
			set { listen_mode = value; }
		}

		public string Name {
			get {
				if (name == null)
					name = Binding.Name + "_" + Contract.Name;
				return name;
			}
			set { name = value; }
		}

		internal void Validate () {
#if !NET_2_1
			foreach (IContractBehavior b in Contract.Behaviors)
				b.Validate (Contract, this);
			foreach (IEndpointBehavior b in Behaviors)
				b.Validate (this);
			foreach (OperationDescription operation in Contract.Operations) {
				foreach (IOperationBehavior b in operation.Behaviors)
					b.Validate (operation);
			}
#endif
		}
	}
}
